package api

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io"
	"io/ioutil"
	"net/http"
	"sku3dweb/conf"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

func AddFassiImage(imgId primitive.ObjectID, url string) error {
	//http请求python
	api := conf.AppConfig.SearchImageAddr + "/add" // 替换为实际的URL

	// 定义参数
	payload := map[string]string{"imageId": imgId.Hex(), "url": url}
	jsonData, err := json.Marshal(payload)
	if err != nil {
		fmt.Println("JSON编码失败:", err)
		return err
	}

	// 发送POST请求
	fmt.Println("add vector=", payload)
	resp, err := http.Post(api, "application/json", bytes.NewBuffer(jsonData))
	fmt.Println("add vector back!!!")

	if err != nil {
		fmt.Println("请求失败:", err)
		return err
	}
	defer resp.Body.Close()
	if resp.StatusCode != http.StatusOK {
		return NewError("fassi 添加失败")
	}

	// 读取响应内容
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		fmt.Println("读取响应失败:", err)
		return err
	}

	// 解析JSON响应
	var result map[string]interface{}
	if err := json.Unmarshal(body, &result); err != nil {
		fmt.Println("JSON解析失败:", err)
		return err
	}
	// 打印JSON响应
	fmt.Println("返回的JSON数据:", result)

	return nil
}

type QueryResp struct {
	Id    primitive.ObjectID
	Score float32
}

func QueryFassiImage(url string, limit int, min_score float64, max_score float64) ([]QueryResp, error) {
	//http请求python
	// 解析JSON响应
	out := []QueryResp{}

	api := conf.AppConfig.SearchImageAddr + "/search" // 替换为实际的URL
	// 定义参数
	payload := map[string]interface{}{"url": url}
	if limit > 0 {
		payload["limit"] = limit
	}
	if min_score > 0 {
		payload["min_score"] = min_score
	}
	if max_score > 0 {
		payload["max_score"] = max_score
	}
	jsonData, err := json.Marshal(payload)
	if err != nil {
		fmt.Println("JSON编码失败:", err)
		return out, err
	}

	// 发送POST请求
	resp, err := http.Post(api, "application/json", bytes.NewBuffer(jsonData))
	if err != nil {
		fmt.Println("请求失败:", err)
		return []QueryResp{}, err
	}
	defer resp.Body.Close()
	if resp.StatusCode != http.StatusOK {
		body, _ := io.ReadAll(resp.Body) // 读取响应体
		fmt.Println("请求失败，状态码:", resp.StatusCode, "响应体:", string(body))
		return []QueryResp{}, NewError("fassi 查询失败")
	}

	// 读取响应内容
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		fmt.Println("读取响应失败:", err)
		return []QueryResp{}, err
	}

	if err := json.Unmarshal(body, &out); err != nil {
		fmt.Println("JSON解析失败:", err)
		return out, err
	}
	// 打印JSON响应
	fmt.Println("返回的JSON数据:", out)
	return out, nil
}

func RomoveFassiImage(id string) (bool, error) {
	out := map[string]string{}

	api := conf.AppConfig.SearchImageAddr + "/remove/" + id // 替换为实际的URL

	// 发送POST请求
	resp, err := http.Post(api, "application/json", nil)
	if err != nil {
		fmt.Println("请求失败:", err)
		return false, err
	}
	defer resp.Body.Close()
	if resp.StatusCode != http.StatusOK {
		body, _ := io.ReadAll(resp.Body) // 读取响应体
		fmt.Println("请求失败，状态码:", resp.StatusCode, "响应体:", string(body))
		return false, NewError("fassi 删除失败")
	}

	// 读取响应内容
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		fmt.Println("读取响应失败:", err)
		return false, err
	}

	if err := json.Unmarshal(body, &out); err != nil {
		fmt.Println("JSON解析失败:", err)
		return false, err
	}

	// 打印JSON响应
	fmt.Println("返回的JSON数据:", out)
	return true, nil
}
